@@ -32,10 +32,14 @@ class UMLS_Entity:
 
     def __init__(self, text):
         self._orig_text = text
+        # The first field that is ignored is the PMID of the document.
         (_, self.start_idx, self.stop_idx, self.mention_text,
-         self.semantic_type_ID, self.concept_ID) = text.split('\t')
+         self.semantic_type_ID, self.cui) = text.split('\t')
+
+        # The CUI may be formatted as "C0123456" or "UMLS:C0123456".
+        # We want to chop off the "UMLS:" if applicable.
+        self.cui = self.cui.split(':')[-1]
 
-        self.concept_ID = self.concept_ID[6:]
         self.start_idx = int(self.start_idx)
         self.stop_idx = int(self.stop_idx)
 
@@ -53,7 +57,11 @@ class PubTatorDocument:
             raw_text (str): simple concatenation of title and abstract. The
                 indexing of characters in raw_text matches the one used in
                 PubTator entity mention annotations.
-            sentences
+            sentences (list<list<str>>): List of sentences, each being a list
+                of tokens.
+            sent_start_end_indices (list<(int, int)>): List of
+                (start_index, end_index) pairs for each sentence (the index
+                refers to the character indices in the original text string)
             start_end_indices
             targets
             text (list<str>): title fused with abstract but as a list of tokens
@@ -80,20 +88,29 @@ class PubTatorDocument:
         # with PubTator PubTator format.
         self.raw_text = self.title + '\n' + self.abstract
         self.umls_entities = [UMLS_Entity(entity) for entity in mentions]
+        self.tokenizer = tokenizer
 
         try:
             sentence_tok = nltk.data.load('tokenizers/punkt/english.pickle')
         except LookupError:
             nltk.download('punkt')
             sentence_tok = nltk.data.load('tokenizers/punkt/english.pickle')
-        self.tokenizer = tokenizer
-        self.sentences = self.raw_text.split('\n')
-        # self.sentences is now of the form ['Title', 'Abstract. Stuff.']
-        self.sentences.extend(sentence_tok.tokenize(self.sentences[1]))
-        # self.sentences is now of the form
-        # ['Title', 'Abstract. Stuff.', 'Abstract.', 'Stuff.']
-        del self.sentences[1]
-        # self.sentences is now of the form ['Title', 'Abstract.', 'Stuff.']
+
+        # We get a list of (start_idx, end_idx) pairs for each sentence
+        self.sent_start_end_indices = list(
+            sentence_tok.span_tokenize(self.raw_text))
+        # the first sentence delimeter is \n at the end of the title, which
+        # the sentence tokenizer does not recognize. Therefore we manually
+        # split the first sentence span.
+        span1_start, span2_end = self.sent_start_end_indices[0]
+        newline = raw_text.index('\n')
+        self.sent_start_end_indices = [
+            (span1_start, newline),
+            (newline + 1, span2_end)] +\
+            self.sent_start_end_indices[1:]
+
+        self.sentences = [self.title]
+        self.sentences.extend(sentence_tok.tokenize(self.abstract))
         self.sentences = [self.tokenizer.tokenize(sentence)
                           for sentence in self.sentences]
         # for example, if tokenization is wordpiece, self.sentences is now
@@ -114,7 +131,7 @@ class PubTatorDocument:
         for i in range(len(char_level_targets)):
             for e in self.umls_entities:
                 if i >= e.start_idx and i < e.stop_idx:
-                    char_level_targets[i] = e.concept_ID, e.semantic_type_ID
+                    char_level_targets[i] = e.cui, e.semantic_type_ID
                     continue
                 elif i > e.stop_idx:
                     continue
